<?php namespace Seller\Lib;
use Dever;
use Dever\Helper\Str;
class Order
{
    # 新增订单
    public function add($info, $goods, $sku, $account, $order)
    {
        $cate = Dever::db('cate', 'goods')->find($goods['cate_id']);
        if ($cate && $cate['rule'] && !preg_match($cate['rule'], $account)) {
            Dever::error($account . '不符合规则');
        }

        # 查询折扣
        $seller_goods = Dever::db('goods', 'seller')->find(array('seller_id' => $info['id'], 'goods_id' => $sku['info_id'], 'sku_id' => $sku['id']));
        if ($seller_goods && $seller_goods['discount'] > 0) {
            $info['discount'] = $seller_goods['discount'];
        }
        if (!$info['discount']) {
            $info['discount'] = 1;
        }
        $cash = round($sku['value'] * $info['discount'], 2);

        # 查询余额是否充足
        $info['yue'] = $info['credit'] + $info['cash'];
        if ($info['yue'] < $cash) {
            Dever::error('余额不足');
        }
        $state = Dever::load('info', 'seller')->dec($info['id'], $cash);

        # 验证订单号
        $check = Dever::db('order', 'seller')->find(array('seller_id' => $info['id'], 'seller_order_num' => $order));
        if ($check) {
            Dever::error('订单重复');
        }

        # 开始下单
        $data = array();
        $data['account'] = $account;
        $data['cate_id'] = $goods['cate_id'];
        $data['goods_id'] = $sku['info_id'];
        $data['sku_id'] = $sku['id'];
        $data['num'] = 1;
        $data['cash'] = $sku['value'];
        $data['price'] = $cash;
        $data['seller_id'] = $info['id'];
        $data['seller_order_num'] = $order;
        $data['seller_request'] = Dever::json_encode(Dever::input());

        $order_id = Dever::db('order', 'seller')->insert($data);
        if (!$order_id) {
            Dever::error('下单失败');
        }
        //\Dever\Helper\Redis::push('submit_' . $data['seller_id'], $order_id);
        $state = \Dever\Helper\Redis::push('submit_' . DEVER_PROJECT, $order_id);
        Dever::load('info', 'seller')->log($info['id'], $order_id, $data['price'], 3);
        return $order_id;
    }

    public function handle($info, $selected = array())
    {
        if ($info['status'] >= 10) {
            return;
        }
        $update['status'] = 2;
        if (!$info['order_num']) {
            $info['order_num'] = $update['order_num'] = $this->createOrder();
        }
        Dever::db('order', 'seller')->update($info['id'], $update);
        # 查找渠道
        $channel_list = Dever::db('channel', 'seller')->select(array('seller_id' => $info['seller_id'], 'goods_id' => $info['goods_id']));
        if (!$channel_list) {
            return $this->notify($info, '通道未开启', $update);
        }
        $channel = $channel_goods = array();
        foreach ($channel_list as $k => $v) {
            if ($selected && isset($selected[$v['channel_id']])) {
                continue;
            }
            if (!$v['sku_id']) {
                $channel_goods = Dever::db('goods', 'channel')->find(array('channel_id' => $v['channel_id'], 'goods_id' => $info['goods_id'], 'sku_id' => $info['sku_id']));
                if ($channel_goods) {
                    $channel = $v['channel_id'];
                    break;
                }
            } else {
                $sku_id = explode(',', $v['sku_id']);
                if (in_array($info['sku_id'], $sku_id)) {
                    $channel = $v['channel_id'];
                    break;
                }
            }
        }
        if (!$channel) {
            return $this->notify($info, '通道未开启', $update);
        }

        $channel = Dever::db('info', 'channel')->find($channel);
        if (!$channel) {
            return $this->notify($info, '通道未开启', $update);
        }
        if ($channel['status'] == 2) {
            return $this->notify($info, '通道未开启', $update);
        }

        # 向渠道发起请求 下单
        $param['order_id'] = $info['id'];
        $param['order_num'] = $info['order_num'];
        $param['account'] = $info['account'];
        $param['cash'] = $info['cash'];
        if ($info['other']) {
            $info['other'] = Dever::json_decode($info['other']);
            $param = array_merge($info['other'], $param);
        }
        if (isset($channel_goods['code'])) {
            $param['code'] = $channel_goods['code'];
        }
        $result = Dever::load('func/api', 'connect')->run(1, $info['cate_id'], $channel, $param);
        $update = array();

        $update['channel_id'] = $channel['id'];
        $update['channel_order_date'] = time();
        $update['channel_request'] = Dever::json_encode($result['request']);
        $update['channel_response'] = Dever::json_encode($result['response']);
        if (isset($channel_goods['id'])) {
            $update['channel_goods_id'] = $channel_goods['id'];
        }
        $update['channel_goods_discount'] = $channel['discount'];
        if (isset($channel_goods['discount']) && $channel_goods['discount']) {
            $update['channel_goods_discount'] = $channel_goods['discount'];
        }
        if (!$update['channel_goods_discount']) {
            $update['channel_goods_discount'] = 1;
        }
        $update['buy_price'] = round($info['cash'] * $update['channel_goods_discount'], 2);
        if ($result['status'] == 1) {
            # 下单成功
            Dever::db('order', 'seller')->update($info['id'], $update);
            return 'ok';
        } else {
            # 下单失败
            return $this->notify($info, '下单失败', $update);
        }
    }

    # 向商户发起回调
    public function notify($info, $msg, $update = array(), $total = 5)
    {
        $seller = Dever::db('info', 'seller')->find($info['seller_id']);
        $status = 11;
        if ($msg == 'ok') {
            $status = 10;
        }
        # 获取商户回调地址，向商户发起回调
        $seller_request = Dever::json_decode($info['seller_request']);
        if (isset($seller_request['notify']) && $seller_request['notify']) {
            $notify = $seller_request['notify'];
        } else {
            $notify = $seller['notify'];
        }
        if ($notify && $info['seller_callback'] != 'ok' && $info['seller_callback_num'] <= $total) {
            $param = array();
            $param['appkey'] = $seller['appkey'];
            $param['order_num'] = $info['seller_order_num'];
            $param['system_order_num'] = $info['order_num'];
            $param['order_status'] = $status;
            $param['order_cash'] = $info['cash'];
            $param['official_order_num'] = $param['official_msg'] = '';
            if (isset($update['official_order_num'])) {
                $param['official_order_num'] = $update['official_order_num'];
            }
            if (isset($update['official_msg'])) {
                $param['official_msg'] = $update['official_msg'];
            }
            $param = \Dever\Helper\Secure::get($param, $seller['appsecret']);
            $response = Dever::curl($notify, $param, 'post')->result();
            # ok是成功
            $update['seller_callback'] = $response;
            $update['seller_callback_date'] = time();
            $update['seller_callback_num'] = $info['seller_callback_num'] + 1;
        }
        $id = $info['id'];
        if ($info['status'] < 10 && $status >= 10) {
            # 订单完成
            $update['status'] = $status;
            $update['finish'] = 1;
            $update['finish_date'] = time();
            if ($status == 11) {
                # 失败，余额加回来
                Dever::load('info', 'seller')->inc($info['seller_id'], $info['price']);
                Dever::load('info', 'seller')->log($info['seller_id'], $info['id'], $info['price'], 4);
            } elseif ($status == 10) {
                # 成功
            }
            # 将记录写到influx中
            $info = array_merge($info, $update);
            Dever::db('order_log', 'seller')->insert($info);
        }
        if (!$info['order_num']) {
            $update['order_num'] = $this->createOrder();
        }
        Dever::db('order', 'seller')->update($id, $update);
        return $msg;
    }

    public function createOrder()
    {
        $where['order_num'] = Str::order('C');
        $state = Dever::db('order', 'seller')->find($where);
        if (!$state) {
            return $where['order_num'];
        } else {
            return $this->createOrder();
        }
    }
}